@@ -1,21 +1,28 @@
 import requests
 import time
 import tempfile
+from requests.adapters import HTTPAdapter
 
-from .config import url_request, url_response, app_key
+from .config import url_request_2captcha, url_response_2captcha, url_request_rucaptcha, url_response_rucaptcha, app_key, \
+    JSON_RESPONSE
 from .errors import RuCaptchaError
+from .result_handler import get_sync_result
 
 
 class RotateCaptcha:
-    def __init__(self, rucaptcha_key, sleep_time=5):
+    def __init__(self, rucaptcha_key: str, service_type: str='2captcha', sleep_time: int=5):
         '''
         Инициализация нужных переменных, создание папки для изображений и кэша
         После завершения работы - удалются временные фалйы и папки
         :param rucaptcha_key:  АПИ ключ капчи из кабинета пользователя
+        :param service_type: Тип сервиса через который будет работать билиотека. Доступны `rucaptcha` или `2captcha`
         :param sleep_time: Вермя ожидания решения капчи
         '''
 
+        if sleep_time < 5:
+            raise ValueError(f'Параметр `sleep_time` должен быть не менее 10. Вы передали - {sleep_time}')
         self.sleep_time = sleep_time
+
         # пайлоад POST запроса на отправку капчи на сервер
         self.post_payload = {"key": rucaptcha_key,
                              'method': 'rotatecaptcha',
@@ -27,27 +34,34 @@ class RotateCaptcha:
                             'action': 'get',
                             'json': 1,
                             }
-        # результат возвращаемый методом *captcha_handler*
-        # в captchaSolve - решение капчи,
-        # в taskId - находится Id задачи на решение капчи, можно использовать при жалобах и прочем,
-        # в errorId - 0 - если всё хорошо, 1 - если есть ошибка,
-        # в errorBody - тело ошибки, если есть.
-        self.result = {"captchaSolve": None,
-                       "taskId": None,
-                       "errorId": None,
-                       "errorBody": None
-                       }
+
+        if service_type == '2captcha':
+            self.url_request = url_request_2captcha
+            self.url_response = url_response_2captcha
+        elif service_type == 'rucaptcha':
+            self.url_request = url_request_rucaptcha
+            self.url_response = url_response_rucaptcha
+        else:
+            raise ValueError('Передан неверный параметр URL-сервиса капчи! Возможные варинты: `rucaptcha` и `2captcha`.'
+                             'Wrong `service_type` parameter. Valid formats: `rucaptcha` or `2captcha`.')
+
+        # создаём сессию
+        self.session = requests.Session()
+        # выставляем кол-во попыток подключения к серверу при ошибке
+        self.session.mount('http://', HTTPAdapter(max_retries=5))
 
     # Работа с капчёй
-    def captcha_handler(self, captcha_link):
+    def captcha_handler(self, captcha_link: str):
         '''
         Метод получает от вас ссылку на изображение, скачивает его, отправляет изображение на сервер
         RuCaptcha, дожидается решения капчи и вовзращает вам результат
         :param captcha_link: Ссылка на изображение
         :return: Ответ на капчу
         '''
+        # результат возвращаемый методом *captcha_handler*
+        self.result = JSON_RESPONSE.copy()
         # Скачиваем изображение
-        content = requests.get(captcha_link).content
+        content = self.session.get(captcha_link).content
         with tempfile.NamedTemporaryFile(suffix='.jpg') as out:
             out.write(content)
             captcha_image = open(out.name, 'rb')
@@ -55,11 +69,11 @@ class RotateCaptcha:
             files = {'file': captcha_image}
             # Отправляем на рукапча изображение капчи и другие парметры,
             # в результате получаем JSON ответ с номером решаемой капчи и получая ответ - извлекаем номер
-            captcha_id = requests.request('POST', url_request, data=self.post_payload, files=files).json()
+            captcha_id = self.session.post(self.url_request, data=self.post_payload, files=files).json()
 
         # если вернулся ответ с ошибкой то записываем её и возвращаем результат
         if captcha_id['status'] is 0:
-            self.result.update({'errorId': 1,
+            self.result.update({'error': True,
                                 'errorBody': RuCaptchaError().errors(captcha_id['request'])
                                 }
                                )
@@ -74,28 +88,8 @@ class RotateCaptcha:
 
         # Ожидаем решения капчи 20 секунд
         time.sleep(self.sleep_time)
-
-        while True:
-            # отправляем запрос на результат решения капчи, если не решена ожидаем
-            captcha_response = requests.post(url_response, data=self.get_payload)
-    
-            # если капча ещё не решена - ожидаем
-            if captcha_response.json()['request'] == 'CAPCHA_NOT_READY':
-                time.sleep(self.sleep_time)
-    
-            # при ошибке во время решения
-            elif captcha_response.json()["status"] == 0:
-                self.result.update({'errorId': 1,
-                                    'errorBody': RuCaptchaError().errors(captcha_response.json()["request"])
-                                    }
-                                   )
-                return self.result
-    
-            # при решении капчи
-            elif captcha_response.json()["status"] == 1:
-                self.result.update({'errorId': 0,
-                                    'captchaSolve': captcha_response.json()['request']
-                                    }
-                                   )
-                return self.result
+        return get_sync_result(get_payload=self.get_payload,
+                               sleep_time = self.sleep_time,
+                               url_response = self.url_response,
+                               result = self.result)
 
